#!/bin/bash
#
# This script updates the display resolution on ISC-178x
#

set -e

usage() {
    cat << EOF

Usage:
$0 [-hqpsv] [-r refresh rate] [-x horizontal resolution]
    [-y vertical resolution]

        -h      display help
        -p      persist
        -q      query the current resolution, returned in the format
                <horizontal resolution>x<vertical resolution> <refresh_rate>
        -r      specify desired refresh rate
        -s      suppress prompts for user input, including confirmation of
                resolution
        -v      enable verbose output
        -x      specify desired horizontal resolution
        -y      specify desired vertical resolution

EOF
}

# Reset index in case getopts had been previously used in the shell
OPTIND=1

persist=0
query=0
suppress=0
verbose=0
x_res=
y_res=
refresh_rate=

while getopts "hpqr:svx:y:" opt; do
    case "$opt" in
    h)
        usage
        exit 0
        ;;
    p)  persist=1
        ;;
    q)  query=1
        ;;
    r)  refresh_rate=$OPTARG
        ;;
    s)  suppress=1
        ;;
    v)  verbose=1
        ;;
    x)  x_res=$OPTARG
        ;;
    y)  y_res=$OPTARG
        ;;
    \?)
        usage
        exit 1
        ;;
    esac
done

# Reset getopts
shift $((OPTIND-1))
[ "$1" = "--" ] && shift

initial_mode="$(xrandr --current | grep \*)"
initial_res="$(echo $initial_mode | awk '{ print $1 }' | cut -f1 -d"_")"
initial_x="$(echo $initial_res | cut -f1 -d"x")"
initial_y="$(echo $initial_res | cut -f2 -d"x")"
initial_refresh_rate="$(echo $initial_mode | awk '{ print $2}' | cut -f1 -d"*")"
# If -q was specified, just provide the current resolution and return
if [ $query -eq 1 ]; then
    echo "$initial_res $initial_refresh_rate"
    exit 0
fi

# If x and y resolutions weren't specified in the arguments, prompt for them
if [ -z "$x_res" ]; then
    if [ $suppress -eq 1 ]; then
        echo ""
        echo When operating with user prompts suppressed, arguments -x \
            and -y are required
        usage
        exit 1
    else
        read -p "Enter target x resolution: " x_res
    fi
fi
if [ -z "$y_res" ]; then
    if [ $suppress -eq 1 ]; then
        echo When operating with user prompts suppressed, arguments -x \
            and -y are required
        usage
        exit 1
    else
        read -p "Enter target y resolution: " y_res
    fi
fi
[ $verbose -eq 1 ] && echo "Updating the resolution to $x_res by $y_res."

# Get the modeline for the desired horizontal and vertical resolutions
if `cvt $x_res $y_res $refresh_rate > /dev/null`; then
    cvt_output="$(cvt $x_res $y_res $refresh_rate | awk 'NR==2')"
    mode="$(echo $cvt_output | grep -o '".*"' | sed 's/"//g')"
    modeline="$(echo $cvt_output | sed 's/.*".*"//')"
    display_output="$(xrandr --current | awk 'NR==2' | sed 's/connected.*//')"
    effective_refresh_rate="$(cvt $x_res $y_res $refresh_rate | \
        awk 'NR==1' | awk '{ print $3 }')"
else
    echo "Error, $x_res x $y_res $refresh_rate is an invalid target resolution"
    exit 1
fi

if [ "$initial_x" == "$x_res" ] && \
   [ "$initial_y" == "$y_res" ] && \
   [ "$initial_refresh_rate" == "effective_refresh_rate" ]; then
    [ $verbose -eq 1 ] && echo The current resolution matches the target \
            resolution, nothing to do.
    exit 0
fi

if ! xrandr --current | grep -q $mode ; then
    # check for pre-defined mode that wouldn't have refresh rate
    # appended to the name
    mode_no_refresh="$(echo $mode | cut -f1 -d"_")"
    refresh_match="$(xrandr --current | grep $mode_no_refresh | \
        awk 'NR==2' | awk '{ print $2 }' | sed 's/*//')"
    if xrandr --current | grep -q $mode_no_refresh && \
        [ "$(xrandr --current | grep $mode_no_refresh | awk 'NR==2' | \
        awk '{ print $2 }' | sed 's/*//')" == "$effective_refresh_rate" ]; then
        mode="$mode_no_refresh"
    else
        xrandr --newmode "$mode" $modeline
        xrandr --addmode $display_output "$mode"
    fi
fi

xrandr --output $display_output --mode "$mode"

if [ $suppress -eq 0 ]; then
    # Prompt user for confirmation, otherwise revert
    echo Please confirm these settings within 10 seconds or the display
    echo will be reverted to its previous configuration.
    if read -t 10 -p "Is this resolution okay? [y/N] " -n 1 -r && \
        [[ $REPLY =~ ^[Yy]$ ]]; then
        echo ""
        [ $verbose -eq 1 ] && echo "$mode confirmed"
    else
        echo ""
        # In some scenarios when the display has been updated, the names of
        # the available modes may not exactly match what had been initially
        # saved off as the initial configuration. Search again here for a
        # match to work around that issue.
        initial_mode_name="$(echo $initial_mode | awk '{ print $1 }')"
        revert_mode="$(xrandr --current | tail -n +3 | \
            grep $initial_mode_name | awk 'NR==1{ print $1 }')"
        [ $verbose -eq 1 ] && echo "Reverting back to $revert_mode"
        xrandr --output $display_output --mode "$revert_mode"
        exit 1
    fi
fi

if [ $persist -eq 1 ]; then
	cat << EOF > /usr/share/X11/xorg.conf.d/40-persist-display.conf
Section "Monitor"
	Identifier "Monitor0"
	Modeline "$mode" $modeline
EndSection

Section "Screen"
	Identifier	"Screen0"
	Device		"$display_output"
	Monitor		"Monitor0"
	SubSection "Display"
		Modes "$mode"
	EndSubSection
EndSection
EOF
fi
